@@ -15,7 +15,7 @@ from .utils import AGGREGATED_OUTPUT_DIR, FILE_METADATA
 class Client:
     def __init__(self,
                  credentials: dict = None,
-                 host: str = "https://api.branchkey.com"):
+                 host: str = "https://api.branchkey.com", ssl: bool = False):
 
         if credentials is None:
             credentials = dict(leaf_name="guest", leaf_id="guest", leaf_session_token="guest",
@@ -33,18 +33,81 @@ class Client:
 
         self.__api_host = host
         # Verify SSL certs
-        self.__verify = True
+        self.__verify = ssl
 
         self.queue = Queue()
-        self.__run_status = 'pause'
+        self.__run_status = None
+        self.__run_number = None
 
         self.rabbit_credentials = dict(
             leaf_name=self.__leaf_name, leaf_id=self.__leaf_id, queue_password=self.__leaf_session_token, host=credentials["response_host"])
         self.consumer = None
+
+        self.__authenticate()
+        self.get_run_details()
         return
 
+    def __authenticate(self):
+        try:
+            url = self.__api_host + "/v1/leaf/authenticate"
+            headers = {"accept": "application/json"}
+
+            resp = requests.get(url, headers=headers,
+                                auth=(self.__leaf_id, self.__leaf_session_token), verify=self.__verify)
+
+            print(f"received resp code form auth: {resp.status_code}")
+            if resp.status_code != HTTPStatus.OK:
+                raise Exception(resp.json())
+
+            authenticate_data = resp.json()["data"]
+            try:
+                self.__status = "authenticated"
+                self.__user_id = authenticate_data["user_id"]
+                self.__tree_id = authenticate_data["tree_id"]
+                self.__branch_id = authenticate_data["branch_id"]
+                self.rabbit_credentials["tree_id"] = authenticate_data["tree_id"]
+                self.rabbit_credentials["branch_id"] = authenticate_data["branch_id"]
+                consumer = Consumer(self.rabbit_credentials,
+                                    self.queue, self.__update_run_status)
+                self.consumer = consumer.spawn_consumer_thread()
+            except Exception as e:
+                raise Exception(
+                    f"Failed to parse response from authenticate API: {e}")
+
+        except Exception as e:
+            msg = "Authentication failed for leaf {}: {}".format(
+                self.__leaf_name, e)
+            raise Exception(msg)
+
+    def get_run_details(self):
+        try:
+            url = self.__api_host + "/v1/branch/" + self.__branch_id + "/run"
+            headers = {"accept": "application/json", "leaf": "true"}
+
+            resp = requests.get(url, headers=headers,
+                                auth=(self.__performance_analyser_username, self.__performance_analyser_password), verify=self.__verify)
+
+            print(
+                f"received resp code from api-gateway for get_run_details: {resp.status_code}")
+            if resp.status_code != HTTPStatus.OK:
+                raise Exception(resp.json())
+
+            run_details = resp.json()["data"]
+            try:
+                self.__run_status = run_details["run_status"]
+                self.__run_number = run_details["run_number"]
+            except Exception as e:
+                raise Exception(
+                    f"Failed to parse response from get_run_details API: {e}")
+
+        except Exception as e:
+            msg = "Getting Run details failed: {}".format(e)
+            raise Exception(msg)
+
     def __update_run_status(self, status):
         self.__run_status = status
+        if status == "start":
+            self.get_run_details()
 
     def disable_ssl_verification(self, enabled=False):
         warnings.warn(
@@ -85,108 +148,68 @@ class Client:
             file.seek(0)
             return True
 
-    def authenticate(self):
+    def file_upload(self, file_path):
         try:
-            url = self.__api_host + "/v1/leaf/authenticate"
-            headers = {"accept": "application/json"}
+            if self.__run_status != "start":
+                raise Exception("run is blocked")
 
-            resp = requests.get(url, headers=headers,
-                                auth=(self.__leaf_id, self.__leaf_session_token), verify=self.__verify)
+            with open(file_path, mode="rb") as f:
+                self.validate_weight_shape(f)
 
-            print(f"received resp code form auth: {resp.status_code}")
-            if resp.status_code != HTTPStatus.OK:
-                raise Exception(resp.json())
+                url = self.__api_host + "/v1/file/upload"
+                headers = {"accept": "application/json"}
+                data = {"data": json.dumps({"leaf_name": self.__leaf_name,
+                                            "metadata": FILE_METADATA})}
+                file = {"file": f}
 
-            authenticate_data = resp.json()["data"]
-            try:
-                self.__status = "authenticated"
-                self.__tree_id = authenticate_data["tree_id"]
-                self.__branch_id = authenticate_data["branch_id"]
-                self.rabbit_credentials["tree_id"] = authenticate_data["tree_id"]
-                self.rabbit_credentials["branch_id"] = authenticate_data["branch_id"]
-                consumer = Consumer(self.rabbit_credentials,
-                                    self.queue, self.__update_run_status)
-                self.consumer = consumer.spawn_consumer_thread()
-                return True
-            except Exception as e:
-                raise Exception(
-                    f"Failed to parse response from authenticate API: {e}")
+                resp = requests.post(
+                    url, files=file, data=data, headers=headers,
+                    auth=(self.__leaf_id, self.__leaf_session_token), verify=self.__verify)
+                f.close()
+
+                if resp.status_code != HTTPStatus.CREATED:
+                    raise Exception(resp.json()["error"])
+                else:
+                    return resp.json()["data"]["file_id"]
 
         except Exception as e:
-            msg = "Authentication failed for leaf {}: {}".format(
-                self.__leaf_name, e)
-            logging.error(msg)
-            return False
-
-    def file_upload(self, file_path):
-        if self.__status == "authenticated":
-            try:
-                with open(file_path, mode="rb") as f:
-                    self.validate_weight_shape(f)
-
-                    url = self.__api_host + "/v1/file/upload"
-                    headers = {"accept": "application/json"}
-                    data = {"data": json.dumps({"leaf_name": self.__leaf_name,
-                                                "metadata": FILE_METADATA})}
-                    file = {"file": f}
-
-                    resp = requests.post(
-                        url, files=file, data=data, headers=headers,
-                        auth=(self.__leaf_id, self.__leaf_session_token), verify=self.__verify)
-                    f.close()
-
-                    if resp.status_code != HTTPStatus.CREATED:
-                        raise Exception(resp.json()["error"])
-                    else:
-                        return resp.json()["data"]["file_id"]
-
-            except Exception as e:
-                msg = "File Upload failed for leaf {} for file {}: {}".format(
-                    self.__leaf_name, file_path, e)
-                raise Exception(msg)
-        else:
-            msg = "File Upload failed for leaf {} : {}".format(
-                self.__leaf_name, "leaf not authenticated")
+            msg = "File Upload failed for leaf {} for file {}: {}".format(
+                self.__leaf_name, file_path, e)
             raise Exception(msg)
 
     def file_download(self, file_id):
-        if self.__status == "authenticated":
+        try:
+            if not os.path.exists(AGGREGATED_OUTPUT_DIR):
+                os.makedirs(AGGREGATED_OUTPUT_DIR)
+
+            url = self.__api_host + "/v1/file/download/" + file_id
+            headers = {"accept": "*/*S"}
+            resp = requests.get(url, headers=headers,
+                                auth=(self.__leaf_id, self.__leaf_session_token), verify=self.__verify)
+
+            if resp.status_code != HTTPStatus.OK:
+                raise Exception(resp.json()["error"])
+
             try:
-                if not os.path.exists(AGGREGATED_OUTPUT_DIR):
-                    os.makedirs(AGGREGATED_OUTPUT_DIR)
-
-                url = self.__api_host + "/v1/file/download/" + file_id
-                headers = {"accept": "*/*S"}
-                resp = requests.get(url, headers=headers,
-                                    auth=(self.__leaf_id, self.__leaf_session_token), verify=self.__verify)
-
-                if resp.status_code != HTTPStatus.OK:
-                    raise Exception(resp.json()["error"])
-
-                try:
-                    download_url = resp.json()["data"]["download_url"]
-                except Exception as e:
-                    raise Exception(
-                        f"Failed to parse response from download API: {resp.json()}")
-
-                resp = requests.get(download_url)
-                if len(resp.content) == 0:
-                    raise Exception("file not received")
-
-                f = open(AGGREGATED_OUTPUT_DIR +
-                         '/' + file_id + ".npy", 'wb')
-                f.write(resp.content)
-                f.close()
-                print(f"downloaded file {file_id}")
-                return True
-
+                download_url = resp.json()["data"]["download_url"]
             except Exception as e:
-                msg = "File Download failed for leaf {} for file {}: {}".format(
-                    self.__leaf_name, file_id, e)
-                raise Exception(msg)
-        else:
-            msg = "File Download failed for leaf {} : {}".format(
-                self.__leaf_name, "leaf not authenticated")
+                raise Exception(
+                    f"Failed to parse response from download API: {resp.json()}")
+
+            resp = requests.get(download_url)
+            if len(resp.content) == 0:
+                raise Exception("file not received")
+
+            f = open(AGGREGATED_OUTPUT_DIR +
+                     '/' + file_id + ".npy", 'wb')
+            f.write(resp.content)
+            f.close()
+            print(f"downloaded file {file_id}")
+            return True
+
+        except Exception as e:
+            msg = "File Download failed for leaf {} for file {}: {}".format(
+                self.__leaf_name, file_id, e)
             raise Exception(msg)
 
     def send_performance_metrics(self, aggregation_id: str, data: str, mode: str):
@@ -220,6 +243,18 @@ class Client:
     def leaf_id(self):
         return self.__leaf_id
 
+    @property
+    def branch_id(self):
+        return self.__branch_id
+
+    @property
+    def tree_id(self):
+        return self.__tree_id
+
+    @property
+    def user_id(self):
+        return self.__user_id
+
     @property
     def session_token(self):
         return self.__leaf_session_token
@@ -231,3 +266,7 @@ class Client:
     @property
     def run_status(self):
         return self.__run_status
+
+    @property
+    def run_number(self):
+        return self.__run_number
